import java.awt.*;
import java.io.*;
import java.awt.datatransfer.*;
import java.awt.event.*;

class Main extends Frame implements ActionListener, ClipboardOwner {
    TextArea textArea = new TextArea();

    Main() {
        super("Clipboard Example");
        MenuBar mb = new MenuBar();
        Menu m = new Menu("Edit");
    
        // Add text area.
        setLayout(new BorderLayout());
        add("Center", textArea);

        // Prepare menu and menubar.
        m.add("Cut");
        m.add("Copy");
        m.add("Paste");
        mb.add(m);
        setMenuBar(mb);

        // Listen to events from the menu items.
        for (int i=0; i<m.getItemCount(); i++) {
            m.getItem(i).addActionListener(this);
        }

        setSize(300, 300);
        show();
    }

    public void actionPerformed(ActionEvent evt) {
        if ("Paste".equals(evt.getActionCommand())) {
            boolean error = true;
            Transferable t = 
                getToolkit().getSystemClipboard().getContents(this);
    
            try {
                if (t != null 
                        && t.isDataFlavorSupported(DataFlavor.stringFlavor)) {
                    textArea.setBackground(Color.white);
                    textArea.setForeground(Color.black);
                    textArea.replaceRange(
                        (String)t.getTransferData(DataFlavor.stringFlavor),
                        textArea.getSelectionStart(),
                        textArea.getSelectionEnd());
                    error = false;
                }
            } catch (UnsupportedFlavorException e) {
            } catch (IOException e) {
            }
            // Display an error message.
            if (error) {
                textArea.setBackground(Color.red);
                textArea.setForeground(Color.white);
                textArea.repaint();
                textArea.setText("ERROR: \nEither the clipboard"
                    + " is empty or the contents is not a string.");
            }
        } else if ("Copy".equals(evt.getActionCommand())) {
            setContents();
        } else if ("Cut".equals(evt.getActionCommand())) {
            setContents();
            textArea.replaceRange("", textArea.getSelectionStart(),
                textArea.getSelectionEnd());
        }
    }

    void setContents() {
        String s = textArea.getSelectedText();

        StringSelection contents = new StringSelection(s);
        getToolkit().getSystemClipboard().setContents(contents, this);
    }

    public void lostOwnership(Clipboard clipboard, Transferable contents) {
        System.out.println("lost ownership");
    }
    public static void main(String args[]) {
        new Main();
    }
}
