import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.io.*;

class Main extends Frame implements ActionListener {
    static Clipboard mainClipboard = new Clipboard("Main Clipboard");

    Main() {
        super("DataFlavor Example");

        // Initialize button.
        Button b = new Button("Paste");
        add(b, BorderLayout.CENTER);
        b.addActionListener(this);

        pack();
        show();
    }

    public void actionPerformed(ActionEvent evt) {
        // Get the clipboard contents.
        Transferable t = mainClipboard.getContents(this);

        // Make sure the clipboard is not empty.
        if (t == null) {
            System.out.println("The clipboard is empty.");
            return;
        }

        try {
            // Retrieve as an integer array.
            int[] ints = (int[])t.getTransferData(
                IntegersSelection.arrayFlavor);
            for (int i=0; i<ints.length; i++) {
                System.out.print(ints[i] + " ");
            }
            System.out.println();

            // Retrieve as an integer stream.
            DataInputStream is = (DataInputStream)t.getTransferData(
                IntegersSelection.streamFlavor);
            try {
                while (true) {
                    System.out.print(is.readInt() + " ");
                }
            } catch (IOException e) {
            }
            System.out.println();

            // Retrieve as a string.
            String s = (String)t.getTransferData(
                IntegersSelection.stringFlavor);
            System.out.println(s);
        } catch (IOException e) {
            System.out.println("The data is no longer available.");
        } catch (UnsupportedFlavorException e) {
            System.out.println("The data flavor is not supported.");
        }
    }

    public static void main(String[] args) {
        new Main();
        new IntegersFrame();
    }
}

class IntegersSelection implements Transferable {
    static DataFlavor arrayFlavor;
    static DataFlavor streamFlavor;
    static DataFlavor stringFlavor;

    int[] ints;
    
    IntegersSelection(int[] ints) {
        this.ints = ints;
    }

    public Object getTransferData(DataFlavor flavor) 
        throws UnsupportedFlavorException, IOException {
        if (flavor.equals(arrayFlavor)) {
            // Return the array itself.
            return ints;
        } else if (flavor.equals(streamFlavor)) {
            // Build a data input stream and return it.
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            DataOutputStream os = new DataOutputStream(bos);
            byte[] buf;
    
            for (int i=0; i<ints.length; i++) {
                os.writeInt(ints[i]);
            }
            return new DataInputStream(
                new ByteArrayInputStream(bos.toByteArray()));
        } else if (flavor.equals(stringFlavor)) {
            // Build a string and return it.
            String s = "";

            for (int i=0; i<ints.length; i++) {
                s += ints[i] + " ";
            }
            return s;
        } else {
            throw new UnsupportedFlavorException(flavor);
        }
    }

    public DataFlavor[] getTransferDataFlavors() {
        return flavors;
    }

    public boolean isDataFlavorSupported(DataFlavor flavor) {
        return flavor.equals(stringFlavor) || flavor.equals(arrayFlavor);
    }

    // Build the data flavors.
    static {
        try {
            streamFlavor = new DataFlavor(Class.forName("java.io.DataInputStream"),
                "Integers in a java.io.DataInputStream");
            arrayFlavor = new DataFlavor(Class.forName("[I"),
                "Integers in an Array");
            stringFlavor = new DataFlavor(Class.forName("java.lang.String"),
                "Integers in a String");
        } catch (ClassNotFoundException e) {
        }
    }

    DataFlavor[] flavors = {arrayFlavor, streamFlavor, stringFlavor};
}

class IntegersFrame extends Frame implements ActionListener {
    List list = new List();

    IntegersFrame() {
        super("DataFlavor Example");

        // Initialize list.
        list.setMultipleMode(true);
        for (int i=0; i<10; i++) {
            list.addItem(""+i);
        }
        add(list, BorderLayout.CENTER);

        // Initialize button.
        Button b = new Button("Copy");
        add(b, BorderLayout.NORTH);
        b.addActionListener(this);

        pack();
        show();
    }

    public void actionPerformed(ActionEvent evt) {
        if (evt.getActionCommand().equals("Copy")) {
            Main.mainClipboard.setContents(
                new IntegersSelection(list.getSelectedIndexes()), null);
        }
    }
}