import java.awt.*;
import java.awt.event.*;

class Main extends Frame {
    String[] words = {"java", "class", "interface", "extends", "implements"};

    // Current mystery word.  Index into 'words'.
    int curWord;

    // Holds current set of correctly guessed letters.
    char[] letters;

    // Holds current set of incorrectly guessed letters.
    String wrongLetters;

    // win is true if all letters have been guessed.
    boolean win;

    Main() {
        super("getKeyChar Example");

        // Listen for text events.
        addKeyListener(new KeyEventHandler());

        newGame();
        setSize(300, 150);
        setFont(new Font("Monospaced", Font.BOLD, 36));
        show();
    }

    void newGame() {
        curWord = (int)Math.floor(Math.random()*words.length);
        letters = new char[words[curWord].length()];
        wrongLetters = "";
        win = false;
    }

    public void paint(Graphics g) {
        Insets in = getInsets();
        FontMetrics fm = g.getFontMetrics();
        int chw = fm.stringWidth("M");

        for (int i=0; i<letters.length; i++) {
            if (letters[i] > 0) {
                g.drawChars(letters, i, 1, in.left+i*(chw+chw/2), 
                            in.top+fm.getAscent());
            }
            g.fillRect(in.left+i*(chw+chw/2), in.top+fm.getHeight(), chw, 2);
        }

        if (win) {
            g.setColor(Color.blue);
            g.drawString("You Won!", in.left, in.top+fm.getHeight() * 2);
        } else {
            g.setColor(Color.red);
            g.drawString(wrongLetters, in.left, in.top+fm.getHeight() * 2);
        }
    }

    class KeyEventHandler extends KeyAdapter {
        public void keyTyped(KeyEvent evt) {
            char ch = evt.getKeyChar();

            if (win) {
                newGame();
            }

            // If not a letter, return.
            ch = Character.toLowerCase(ch);
            if (!Character.isLetter(ch)) {
                getToolkit().beep();
                return;
            }

            // Has the letter been guessed?
            for (int i=0; i<letters.length; i++) {
                if (ch == letters[i]) {
                    getToolkit().beep();
                    return;
                }
            }

            // Is the letter already in the wrong letter list?
            if (wrongLetters.indexOf(ch) >= 0) {
                getToolkit().beep();
                return;
            }

            // Is the letter correct?
            if (words[curWord].indexOf(ch) >= 0) {
                for (int i=0; i<letters.length; i++) {
                    if (words[curWord].charAt(i) == ch) {
                        letters[i] = ch;
                    }
                }

                boolean foundBlank = false;
                for (int i=0; i<letters.length; i++) {
                    if (letters[i] == 0) {
                        foundBlank = true;
                    }
                }
                if (!foundBlank) {
                    win = true;
                }
            } else {
                wrongLetters += ch;
            }
            repaint();
        }
    }
    
    public static void main(String args[]) {
        new Main();
    }
}
