import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.net.*;
import java.util.*;

class Main extends Frame {
    Main(String filename) {
        super("FilteredImageSource Example");
        try {
            Image image = getToolkit().getImage(filename);

            // Use a media tracker object to wait until all the pixels 
            // have been retrieved.
            MediaTracker tracker = new MediaTracker(this);
            tracker.addImage(image, 0);
            tracker.waitForID(0);

            add(new ImageCanvas(image), BorderLayout.CENTER);
        } catch (Exception e) {
            e.printStackTrace();
        }
        setSize(50, 100);
        show();
    }

    static public void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("usage: java Main <image file>");
        }
    }
}

class ImageCanvas extends Component {
    Image newImage;

    ImageCanvas(Image image) {
        ImageProducer ip = image.getSource();

        ip = new FilteredImageSource(ip, new CropImageFilter(0, 0, 
                        image.getWidth(this), image.getHeight(this)*3/4));
        ip = new FilteredImageSource(ip, new RedBlueSwapFilter());
        newImage = getToolkit().createImage(ip);
    }

    public void paint(Graphics g) {
        g.drawImage(newImage, 0, 0, this);
    }
}

class RedBlueSwapFilter extends RGBImageFilter {
    public RedBlueSwapFilter() {
        canFilterIndexColorModel = true;
    }

    public int filterRGB(int x, int y, int rgb) {
        return ((rgb & 0xff00ff00)
                | ((rgb & 0xff0000) >> 16)
                | ((rgb & 0xff) << 16));
    }
}
