import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

class Main extends Frame {
    BlockImageDecoder ip;
    
    Main(String filename) {
        super("ImageProducer Example");
        Panel p = new Panel();
        
        try {
            ip = new BlockImageDecoder(new FileInputStream(filename));
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(1);
        }
        p.setLayout(new GridLayout(1, 0));
        p.add(new ImageCanvas(ip));
        p.add(new ImageCanvas(ip));
        p.add(new ImageCanvas(ip));
        add(p, BorderLayout.CENTER);

        setSize(325, 220);
        show();
    }

    public static void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("usage: java Main <input file>");
        }
    }
}

class ImageCanvas extends Component {
    ImageProducer ip;
    Image image;

    ImageCanvas(ImageProducer ip) {
        this.ip = ip;
        image = createImage(ip);
        addMouseListener(new MouseEventHandler());
    }

    public void paint(Graphics g) {
        g.drawImage(image, 0, 0, this);
    }

    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            image = createImage(ip);
            repaint();
        }
    }
}

class BlockImageDecoder implements ImageProducer {
    StreamTokenizer st;
    ColorModel model;
    Vector consumers = new Vector();
    Hashtable properties = new Hashtable();
    byte[] pixels;
    int width = -1, height = -1;
    Rectangle[] rects;
    byte[] rectColors;


    BlockImageDecoder(InputStream is) {
        Reader r = new BufferedReader(new InputStreamReader(is));
        st = new StreamTokenizer(r);
    }

    public synchronized void addConsumer(ImageConsumer ic) {
        if (rects != null) {
            produce(ic);
        } else if (!consumers.contains(ic)) {
            consumers.addElement(ic);
        }
    }

    public synchronized boolean isConsumer(ImageConsumer ic) {
        return consumers.contains(ic);
    }

    public synchronized void removeConsumer(ImageConsumer ic) {
        consumers.removeElement(ic);
    }

    private int getInt() throws IOException {
        if (st.nextToken() != StreamTokenizer.TT_NUMBER) {
            throw (new IOException("format error"));
        }
        return (int)st.nval;
    }

    // Convert the contents of the input file to image data.
    private synchronized void processImage() {
        try {
            // Read width and height
            width = getInt();
            height = getInt();

            // Read colors
            int numColors = getInt();
            byte cmap[] = new byte[numColors * 3];
            for (int i=0; i<cmap.length; i++) {
                cmap[i] = (byte)getInt();
            }

            // Create color model.
            int n = numColors-1;
            int nbits = 0;
            while (n > 0) {
                n >>>= 1;
                nbits++;
            }
            model = new IndexColorModel(nbits, numColors, cmap, 0, false, 0);

            // Read rectangles and their colors
            rects = new Rectangle[getInt()];
            rectColors = new byte[rects.length];

            for (int i=0; i<rects.length; i++) {
                rects[i] = new Rectangle(getInt(), getInt(), getInt(), getInt());
                rectColors[i] = (byte)getInt();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void startProduction(ImageConsumer ic) {
        if (ic != null) {
            addConsumer(ic);
            if (rects == null) {
                processImage();
            }
            for (int i=consumers.size()-1; i>=0; i--) {
                ic = (ImageConsumer)consumers.elementAt(i);
                produce(ic);
                consumers.removeElement(ic);
            }
        }
    }

    private void produce(ImageConsumer ic) {
        if (width < 0) {
            ic.imageComplete(ImageConsumer.IMAGEERROR);
            return;
        } 
        ic.setDimensions(width, height);
        ic.setProperties(properties);
        ic.setColorModel(model);
        ic.setHints(ImageConsumer.SINGLEPASS |
                    ImageConsumer.SINGLEFRAME);
        pixels = new byte[width * height];
        ic.setPixels(0, 0, width, height, model, pixels, 0, width);
        for (int i=0; i<rects.length; i++) {
            Rectangle r = rects[i];

            for (int y=r.y; y<r.y+r.height; y++) {
                for (int x=r.x; x<r.x+r.width; x++) {
                    pixels[y*width+x] = rectColors[i];
                }
            }
            ic.setPixels(r.x, r.y, r.width, r.height, model,
                         pixels, r.x+r.y*width, width);
        }
        ic.imageComplete(ImageConsumer.STATICIMAGEDONE);
    }

    public void requestTopDownLeftRightResend(ImageConsumer ic) {
        if (pixels != null) {
            ic.setDimensions(width, height);
            ic.setProperties(properties);
            ic.setColorModel(model);
            ic.setHints(ImageConsumer.SINGLEPASS |
                        ImageConsumer.TOPDOWNLEFTRIGHT |
                        ImageConsumer.SINGLEFRAME);
            ic.setPixels(0, 0, width, height, model, pixels, 0, width);
        }
    }
}
