import java.awt.*;
import java.awt.image.*;
import java.util.*;

class Main extends Frame {
    Main(String filename) {
        super("RGBFilter Example");
        add(new ImageCanvas(getToolkit().getImage(filename)), 
            BorderLayout.CENTER);

        setSize(50, 100);
        show();
    }

    static public void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("usage: java Main <image file>");
        }
    }
}

class ImageCanvas extends Component {
    Image gradientImage;

    ImageCanvas(Image image) {
        GradientFilter imgf = new GradientFilter();
        ImageProducer ip = image.getSource();
        ip = new FilteredImageSource(ip, imgf);
        gradientImage = getToolkit().createImage(ip);
        repaint();
    }

    public void paint(Graphics g) {
        g.drawImage(gradientImage, 0, 0, this);
    }
}

class GradientFilter extends RGBImageFilter {
    float[] hsb = new float[3];
    int width, height;

    public void setDimensions(int w, int h) {
        super.setDimensions(w, h);
        width = w;
        height = h;
    }

    public int filterRGB(int x, int y, int rgb) {
        Color c = new Color(rgb);
        Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), hsb);
        hsb[2] += .5f - (float)x / width;
        hsb[2] += .5f - (float)y / height;
        hsb[2] = Math.max(0.0f, Math.min(1.0f, hsb[2]));
        return Color.HSBtoRGB(hsb[0], hsb[1], hsb[2]);
    }
}

