import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;

public class MapCustomizer extends Panel 
       implements ActionListener, ItemListener, Customizer {
    Bean bean;

    // Components
    Button newBtn = new Button("New");
    Button deleteBtn = new Button("Delete");
    TextField labelTf = new TextField(15);
    Button saveBtn = new Button("Save");

    // Offscreen image for double-buffering.
    transient Image bbuf;

    Map map;

    public MapCustomizer() {
        map = new Map();
        map.inCustomizer = true;

        // Layout buttons.
        Panel p = new Panel(new GridLayout(0, 1));
        p.add(labelTf);
        p.add(newBtn);
        p.add(deleteBtn);
        p.add(saveBtn);
        Panel q = new Panel(new BorderLayout());
        q.add(p, BorderLayout.NORTH);
    
        // Layout map and buttons.
        setLayout(new BorderLayout());
        add(q, BorderLayout.EAST);
        add(map, BorderLayout.CENTER);

        // Listen for events.
        map.addItemListener(this);
        labelTf.addActionListener(this);
        newBtn.addActionListener(this);
        deleteBtn.addActionListener(this);
        saveBtn.addActionListener(this);
    }

    public void setObject(Object b) {
        bean = (Bean)b;
        map.setAreas(bean.getAreas());
        map.setImage(bean.getImage());
    }

    // Action event handler.
    public void actionPerformed(ActionEvent evt) {
        if (evt.getSource() == newBtn) {
            // Add a new entry.
            Vector v = Area.toVector(map.areas);
            Area a = new Area(new Rectangle(0, 0, 30, 30), "Unknown");
            v.addElement(a);
            map.setAreas(Area.fromVector(v));
        } else if (evt.getSource() == deleteBtn) {
            if (map.curArea >= 0) {
                // Delete the current entry.
                Vector v = Area.toVector(map.areas);
                v.removeElementAt(map.curArea);
                map.setAreas(Area.fromVector(v));
            }
        } else if (evt.getSource() == saveBtn) {
            Area[] oldAreas = bean.getAreas();
            bean.setAreas(map.areas);
            pceListeners.firePropertyChange("areas", oldAreas, map.areas);
        } else if (evt.getSource() == labelTf) {
            map.areas[map.curArea].label = labelTf.getText();
        }
        repaint();
    }

    // Item event handler.
    public void itemStateChanged(ItemEvent evt) {
        labelTf.setText(map.areas[((Integer)evt.getItem()).intValue()].label);
    }

    // Create the listener list.
    PropertyChangeSupport pceListeners = 
        new PropertyChangeSupport(this);

    // The listener list wrapper methods.
    public synchronized void addPropertyChangeListener(
                                PropertyChangeListener l) {
        pceListeners.addPropertyChangeListener(l);
    }
    
    public synchronized void removePropertyChangeListener(
                                   PropertyChangeListener l) {
        pceListeners.removePropertyChangeListener(l);
    }
}
