import java.awt.*;
import java.beans.*;
import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import java.net.*;

class Main {
    public static void main(String[] args) {
        if (args.length != 2) {
            System.err.println(
                "Usage: java Main <bean class file> <output file>.ser");
            System.exit(1);
        }
        if (!args[1].endsWith(".ser")) {
            System.err.println("Output file must end with .ser");
            System.exit(1);
        }

        try {
            // Load the bean and get the bean-info.
            // (See the Introspector class example for 
            // FileClassLoader source code.)
            Class beanClass = FileClassLoader.load(args[0]);
            BeanInfo beanInfo = Introspector.getBeanInfo(beanClass);

            // Instantiate the bean.
            Object bean = beanClass.newInstance(); 

            // Find all color properties.
            PropertyDescriptor pds[] = beanInfo.getPropertyDescriptors();
            for (int i=0; i<pds.length; i++) {
                if (pds[i].getPropertyType() == Color.class) {
                    // Create a random color.
                    Color color = new Color(Color.HSBtoRGB(
                        (float)Math.random(), (float)Math.random(), 
                        (float)Math.random()));

                    // Get the write method.
                    Method writer = pds[i].getWriteMethod();
    
                    // Invoke the write accessor method.
                    Object[] arguments = {color};
                    writer.invoke(bean, arguments);
                    
                    // Print the results.
                    System.out.println(pds[i].getName() + ":    " + color);
                }
            }

            // Serialize the bean.
            java.io.FileOutputStream fos = 
                new java.io.FileOutputStream(args[1]);
            java.io.ObjectOutput os = 
                new java.io.ObjectOutputStream(fos);
            os.writeObject(bean);
            os.close();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (IntrospectionException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (InstantiationException e) {
            e.printStackTrace();
        }

        System.exit(0);
    }
}
/* 
    command to run example:
        java Main Bean.class random-color-bean.ser
    manifext.txt:
        Name: random-color-bean.ser
        Java-Bean: True
    jar command:
        jar cfm bean.jar manifest.txt *.ser
*/
